@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-gst.c,v 1.119 2004/12/24 17:41:34 hadess Exp $
+ * $Id: bacon-video-widget-gst.c,v 1.135 2005/01/18 14:30:17 rbultje Exp $
  *
  */
 
@@ -33,6 +33,7 @@
 #include <gst/colorbalance/colorbalance.h>
 
 /* system */
+#include <time.h>
 #include <string.h>
 #include <stdio.h>
 
@@ -40,6 +41,7 @@
 #include <gdk/gdkx.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <gconf/gconf-client.h>
 
 #include "bacon-video-widget.h"
 #include "baconvideowidget-marshal.h"
@@ -87,6 +89,13 @@ enum
   PROP_SHOW_VISUALS
 };
 
+static char *video_props_str[4] = {
+	GCONF_PREFIX"/brightness",
+	GCONF_PREFIX"/contrast",
+	GCONF_PREFIX"/saturation",
+	GCONF_PREFIX"/hue"
+};
+
 struct BaconVideoWidgetPrivate
 {
   double display_ratio;
@@ -137,12 +146,19 @@ struct BaconVideoWidgetPrivate
   
   char *mrl;
   char *media_device;
+
+  BaconVideoWidgetAudioOutType speakersetup;
+  TvOutType tv_out_type;
+  gint connection_speed;
+
+  GConfClient *gc;
 };
 
 enum {
   ASYNC_VIDEO_SIZE,
   ASYNC_ERROR,
   ASYNC_FOUND_TAG,
+  ASYNC_CLEAR_TAGS,
   ASYNC_EOS,
   ASYNC_BUFFERING
 };
@@ -333,6 +349,111 @@ bacon_video_widget_expose_event (GtkWidget *widget, GdkEventExpose *event)
   return TRUE;
 }
 
+static gboolean
+bacon_video_widget_motion_notify (GtkWidget *widget,
+				  GdkEventMotion *event)
+{
+  BaconVideoWidget *bvw = BACON_VIDEO_WIDGET (widget);
+
+  g_return_val_if_fail (bvw->priv->play != NULL, FALSE);
+
+  if (bvw->priv->media_has_video) {
+    GstElement *videosink = NULL;
+
+    g_object_get (G_OBJECT (bvw->priv->play), "video-sink", &videosink, NULL);
+    if (GST_IS_BIN (videosink)) {
+      videosink = gst_bin_get_by_interface (GST_BIN (videosink),
+          GST_TYPE_NAVIGATION);
+    }
+
+    if (videosink && GST_IS_NAVIGATION (videosink) &&
+        GST_STATE (videosink) >= GST_STATE_PAUSED) {
+      GstNavigation *nav = GST_NAVIGATION (videosink);
+
+      gst_navigation_send_mouse_event (nav,
+          "mouse-move", 0, event->x, event->y);
+
+      /* we need a return value... */
+      return TRUE;
+    }
+  }
+
+  if (GTK_WIDGET_CLASS (parent_class)->motion_notify_event)
+    return GTK_WIDGET_CLASS (parent_class)->motion_notify_event (widget, event);
+
+  return FALSE;
+}
+
+static gboolean
+bacon_video_widget_button_press (GtkWidget *widget,
+				 GdkEventButton *event)
+{
+  BaconVideoWidget *bvw = BACON_VIDEO_WIDGET (widget);
+
+  g_return_val_if_fail (bvw->priv->play != NULL, FALSE);
+
+  if (bvw->priv->media_has_video) {
+    GstElement *videosink = NULL;
+
+    g_object_get (G_OBJECT (bvw->priv->play), "video-sink", &videosink, NULL);
+    if (GST_IS_BIN (videosink)) {
+      videosink = gst_bin_get_by_interface (GST_BIN (videosink),
+         GST_TYPE_NAVIGATION);
+    }
+
+    if (videosink && GST_IS_NAVIGATION (videosink) &&
+        GST_STATE (videosink) >= GST_STATE_PAUSED) {
+      GstNavigation *nav = GST_NAVIGATION (videosink);
+
+      gst_navigation_send_mouse_event (nav,
+          "mouse-button-press", event->button, event->x, event->y);
+
+      /* we need a return value... */
+      return TRUE;
+    }
+  }
+
+  if (GTK_WIDGET_CLASS (parent_class)->button_press_event)
+    return GTK_WIDGET_CLASS (parent_class)->button_press_event (widget, event);
+
+  return FALSE;
+}
+
+static gboolean
+bacon_video_widget_button_release (GtkWidget *widget,
+				   GdkEventButton *event)
+{
+  BaconVideoWidget *bvw = BACON_VIDEO_WIDGET (widget);
+
+  g_return_val_if_fail (bvw->priv->play != NULL, FALSE);
+
+  if (bvw->priv->media_has_video) {
+    GstElement *videosink = NULL;
+
+    g_object_get (G_OBJECT (bvw->priv->play), "video-sink", &videosink, NULL);
+    if (GST_IS_BIN (videosink)) {
+      videosink = gst_bin_get_by_interface (GST_BIN (videosink),
+          GST_TYPE_NAVIGATION);
+    }
+
+    if (videosink && GST_IS_NAVIGATION (videosink) &&
+        GST_STATE (videosink) >= GST_STATE_PAUSED) {
+      GstNavigation *nav = GST_NAVIGATION (videosink);
+
+      gst_navigation_send_mouse_event (nav,
+          "mouse-button-release", event->button, event->x, event->y);
+
+      /* we need a return value... */
+      return TRUE;
+    }
+  }
+
+  if (GTK_WIDGET_CLASS (parent_class)->button_release_event)
+    return GTK_WIDGET_CLASS (parent_class)->button_release_event (widget, event);
+
+  return FALSE;
+}
+
 static void
 bacon_video_widget_size_request (GtkWidget * widget,
 				 GtkRequisition * requisition)
@@ -412,6 +533,9 @@ bacon_video_widget_class_init (BaconVideoWidgetClass * klass)
   widget_class->size_allocate = bacon_video_widget_size_allocate;
   widget_class->realize = bacon_video_widget_realize;
   widget_class->expose_event = bacon_video_widget_expose_event;
+  widget_class->motion_notify_event = bacon_video_widget_motion_notify;
+  widget_class->button_press_event = bacon_video_widget_button_press;
+  widget_class->button_release_event = bacon_video_widget_button_release;
 
   /* FIXME:
    * - once I've re-added DVD supports, I want to add event handlers
@@ -579,6 +703,13 @@ bacon_video_widget_signal_idler (BaconVideoWidget *bvw)
           } else {
             bacon_video_widget_size_allocate (GTK_WIDGET (bvw),
 					      &GTK_WIDGET (bvw)->allocation);
+
+	    /* Uhm, so this ugly hack here makes media loading work for
+	     * weird laptops with NVIDIA graphics cards... Dunno what the
+	     * bug is really, but hey, it works. :). */
+	    gdk_window_hide (GTK_WIDGET (bvw)->window);
+	    gdk_window_show (GTK_WIDGET (bvw)->window);
+
             bacon_video_widget_expose_event (GTK_WIDGET (bvw), NULL);
           }
 
@@ -636,6 +767,14 @@ bacon_video_widget_signal_idler (BaconVideoWidget *bvw)
 			 0, NULL);
           break;
         }
+      case ASYNC_CLEAR_TAGS:
+        {
+          gst_tag_list_free (bvw->priv->tagcache);
+          bvw->priv->tagcache = NULL;
+          g_signal_emit (G_OBJECT (bvw),
+			 bvw_table_signals[SIGNAL_GOT_METADATA], 0, NULL);
+          break;
+        }
       case ASYNC_EOS:
         {
           gst_element_set_state (GST_ELEMENT (bvw->priv->play),
@@ -678,6 +817,22 @@ got_found_tag (GstElement *play, GstElement *source,
   g_idle_add ((GSourceFunc) bacon_video_widget_signal_idler, bvw);
 }
 
+static void
+group_switch (GstElement *play, BaconVideoWidget *bvw)
+{
+  BVWSignal *signal;
+
+  g_return_if_fail (bvw != NULL);
+  g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+
+  signal = g_new0 (BVWSignal, 1);
+  signal->signal_id = ASYNC_CLEAR_TAGS;
+
+  g_async_queue_push (bvw->priv->queue, signal);
+
+  g_idle_add ((GSourceFunc) bacon_video_widget_signal_idler, bvw);
+}
+
 static void
 got_video_size (GstElement * play, guint width, guint height,
 		BaconVideoWidget * bvw)
@@ -802,22 +957,14 @@ cb_iterate (BaconVideoWidget *bvw)
 {
   GstFormat fmt = GST_FORMAT_TIME;
   gint64 value;
-  gboolean res;
-
-  if (!GST_FLAG_IS_SET (bvw->priv->play, GST_BIN_SELF_SCHEDULABLE)) {
-    res = gst_bin_iterate (GST_BIN (bvw->priv->play));
-  } else {
-    g_usleep (100);
-    res = (gst_element_get_state (bvw->priv->play) == GST_STATE_PLAYING);
-  }
 
   /* check length/pos of stream */
-  if (!bvw->priv->stream_length) {
-    if (gst_element_query (GST_ELEMENT (bvw->priv->play),
-			   GST_QUERY_TOTAL, &fmt, &value)) {
-      got_stream_length (GST_ELEMENT (bvw->priv->play),
-			 value, bvw);
-    }
+  if (gst_element_query (GST_ELEMENT (bvw->priv->play),
+			 GST_QUERY_TOTAL, &fmt, &value) &
+      GST_CLOCK_TIME_IS_VALID (value) &&
+      value / GST_MSECOND != bvw->priv->stream_length) {
+    got_stream_length (GST_ELEMENT (bvw->priv->play),
+		       value, bvw);
   }
   if (gst_element_query (GST_ELEMENT (bvw->priv->play),
 			 GST_QUERY_POSITION, &fmt, &value)) {
@@ -825,10 +972,7 @@ cb_iterate (BaconVideoWidget *bvw)
 		   value, bvw);
   }
 
-  if (!res)
-    bvw->priv->update_id = 0;
-
-  return res;
+  return TRUE;
 }
 
 static void
@@ -876,7 +1020,7 @@ state_change (GstElement *play, GstElementState old_state,
   } else if (new_state == GST_STATE_PLAYING) {
     if (bvw->priv->update_id != 0)
       g_source_remove (bvw->priv->update_id);
-    bvw->priv->update_id = g_idle_add ((GSourceFunc) cb_iterate, bvw);
+    bvw->priv->update_id = g_timeout_add (200, (GSourceFunc) cb_iterate, bvw);
   }
 
   if (old_state <= GST_STATE_READY &&
@@ -1056,44 +1200,120 @@ bacon_video_widget_get_backend_name (BaconVideoWidget * bvw)
 int
 bacon_video_widget_get_subtitle (BaconVideoWidget * bvw)
 {
-  return -1;
+  int subtitle = -1;
+
+  g_return_val_if_fail (bvw != NULL, -2);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), -2);
+  g_return_val_if_fail (bvw->priv->play != NULL, -2);
+
+  g_object_get (G_OBJECT (bvw->priv->play), "current-text", &subtitle, NULL);
+
+  if (subtitle == -1)
+    subtitle = -2;
+
+  return subtitle;
 }
 
 void
 bacon_video_widget_set_subtitle (BaconVideoWidget * bvw, int subtitle)
 {
+  g_return_if_fail (bvw != NULL);
+  g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+  g_return_if_fail (bvw->priv->play != NULL);
+
+  if (subtitle == -1)
+    subtitle = 0;
+  else if (subtitle == -2)
+    subtitle = -1;
+
+  g_object_set (G_OBJECT (bvw->priv->play), "current-text", subtitle, NULL);
+}
+
+static GList *
+get_list_of_type (BaconVideoWidget * bvw, const gchar * type_name)
+{
+  GList *streaminfo, *ret = NULL;
+  gint num = 0;
+
+  g_object_get (G_OBJECT (bvw->priv->play), "stream-info", &streaminfo, NULL);
+  for ( ; streaminfo != NULL; streaminfo = streaminfo->next) {
+    GObject *info = streaminfo->data;
+    gint type;
+    GParamSpec *pspec;
+    GEnumValue *val;
+
+    g_object_get (info, "type", &type, NULL);
+    pspec = g_object_class_find_property (G_OBJECT_GET_CLASS (info), "type");
+    val = g_enum_get_value (G_PARAM_SPEC_ENUM (pspec)->enum_class, type);
+
+    if (strstr (val->value_name, type_name)) {
+      ret = g_list_prepend (ret, g_strdup_printf ("%s %d", type_name, num++));
+    }
+  }
+
+  return g_list_reverse (ret);
 }
 
 GList * bacon_video_widget_get_subtitles (BaconVideoWidget * bvw)
 {
-  return NULL;
+  return get_list_of_type (bvw, "TEXT");
 }
 
 GList * bacon_video_widget_get_languages (BaconVideoWidget * bvw)
 {
-  return NULL;
+  return get_list_of_type (bvw, "AUDIO");
 }
 
 int
 bacon_video_widget_get_language (BaconVideoWidget * bvw)
 {
-  return -1;
+  int language = -1;
+
+  g_return_val_if_fail (bvw != NULL, -2);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), -2);
+  g_return_val_if_fail (bvw->priv->play != NULL, -2);
+
+  g_object_get (G_OBJECT (bvw->priv->play), "current-audio", &language, NULL);
+
+  if (language == -1)
+    language = -2;
+
+  return language;
 }
 
 void
 bacon_video_widget_set_language (BaconVideoWidget * bvw, int language)
 {
+  g_return_if_fail (bvw != NULL);
+  g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+  g_return_if_fail (bvw->priv->play != NULL);
+
+  if (language == -1)
+    language = 0;
+  else if (language == -2)
+    language = -1;
+
+  g_object_set (G_OBJECT (bvw->priv->play), "current-audio", language, NULL);
 }
 
 int
 bacon_video_widget_get_connection_speed (BaconVideoWidget * bvw)
 {
-  return 0;
+  g_return_val_if_fail (bvw != NULL, 0);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), 0);
+
+  return bvw->priv->connection_speed;
 }
 
 void
 bacon_video_widget_set_connection_speed (BaconVideoWidget * bvw, int speed)
 {
+  g_return_if_fail (bvw != NULL);
+  g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+
+  bvw->priv->connection_speed = speed;
+  gconf_client_set_int (bvw->priv->gc,
+       GCONF_PREFIX"/connection_speed", speed, NULL);
 }
 
 void
@@ -1111,26 +1331,155 @@ bacon_video_widget_get_deinterlacing (BaconVideoWidget * bvw)
 gboolean
 bacon_video_widget_set_tv_out (BaconVideoWidget * bvw, TvOutType tvout)
 {
-  return FALSE;
+  g_return_val_if_fail (bvw != NULL, FALSE);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), FALSE);
+
+  bvw->priv->tv_out_type = tvout;
+  gconf_client_set_int (bvw->priv->gc,
+      GCONF_PREFIX"/tv_out_type", tvout, NULL);
+
+  return TRUE;
 }
 
 TvOutType
 bacon_video_widget_get_tv_out (BaconVideoWidget * bvw)
 {
-  return TV_OUT_NONE;
+  g_return_val_if_fail (bvw != NULL, 0);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), 0);
+
+  return bvw->priv->tv_out_type;
+}
+
+static GstCaps *
+cb_audio_fixate (GstPad *pad,
+		 const GstCaps *in_caps,
+		 BaconVideoWidget *bvw)
+{
+  gint channels = -1, n, count, diff = -1;
+  const GstStructure *s, *closest = NULL;
+  const GValue *v;
+
+  switch (bvw->priv->speakersetup) {
+    case BVW_AUDIO_SOUND_STEREO:
+      channels = 2;
+      break;
+    case BVW_AUDIO_SOUND_4CHANNEL:
+      channels = 4;
+      break;
+    case BVW_AUDIO_SOUND_5CHANNEL:
+      channels = 5;
+      break;
+    case BVW_AUDIO_SOUND_41CHANNEL:
+      /* so alsa has this as 5.1, but empty center speaker. We don't really
+       * do that yet. ;-). So we'll take the placebo approach. */
+    case BVW_AUDIO_SOUND_51CHANNEL:
+      channels = 6;
+      break;
+    case BVW_AUDIO_SOUND_AC3PASSTHRU:
+    default:
+      break;
+  }
+
+  if (channels == -1)
+    return NULL;
+
+  /* there might be multiple structures, and one might contain the
+   * channelcount that we want. Try that. */
+  count = gst_caps_get_size (in_caps);
+  for (n = 0; n < count; n++) {
+    s = gst_caps_get_structure (in_caps, n);
+    v = gst_structure_get_value (s, "channels");
+    if (!v)
+      continue;
+
+    /* get channel count (or list of ~) */
+    if (G_VALUE_TYPE (v) == G_TYPE_INT) {
+      gint c = g_value_get_int (v);
+
+      if (channels == c) {
+        if (count == 1)
+          return NULL;
+        return gst_caps_new_full (gst_structure_copy (s), NULL);
+      } else if (!closest || abs (c - channels) < diff) {
+        diff = abs (c - channels);
+        closest = s;
+      }
+    } else if (G_VALUE_TYPE (v) == GST_TYPE_INT_RANGE) {
+      gint c1 = gst_value_get_int_range_min (v),
+           c2 = gst_value_get_int_range_max (v);
+
+      if (c1 <= channels  && c2 >= channels) {
+        GstCaps *caps;
+
+        caps = gst_caps_new_full (gst_structure_copy (s), NULL);
+        gst_caps_structure_fixate_field_nearest_int (
+            gst_caps_get_structure (caps, 0), "channels", channels);
+
+        return caps;
+      } else if (!closest ||
+                 (c1 > channels && c1 - channels < diff) ||
+                 (c2 < channels && channels - c2 < diff)) {
+        closest = s; /* FIXME: fixate already? */
+        diff = c1 > channels ? c1 - channels : channels - c2;
+      }
+    } else if (G_VALUE_TYPE (v) == GST_TYPE_LIST) {
+      const GValue *kid;
+      gint nkid, kidcount = gst_value_list_get_size (v), kidc;
+
+      for (nkid = 0; nkid < kidcount; nkid++) {
+        kid = gst_value_list_get_value (v, nkid);
+        if (G_VALUE_TYPE (kid) != G_TYPE_INT)
+          continue;
+        kidc = g_value_get_int (kid);
+        if (kidc == channels) {
+          GstCaps *caps;
+
+          caps = gst_caps_new_full (gst_structure_copy (s), NULL);
+          gst_caps_structure_fixate_field_nearest_int (
+              gst_caps_get_structure (caps, 0), "channels", channels);
+
+          return caps;
+        } else if (!closest || abs (kidc - channels) < diff) {
+          diff = abs (kidc - channels);
+          closest = s;
+        }
+      }
+    }
+  }
+
+  /* FIXME: we could do something with closest here... */
+  return NULL;
 }
 
 BaconVideoWidgetAudioOutType
 bacon_video_widget_get_audio_out_type (BaconVideoWidget *bvw)
 {
-  return BVW_AUDIO_SOUND_STEREO;
+  g_return_val_if_fail (bvw != NULL, -1);
+  g_return_val_if_fail (BACON_IS_VIDEO_WIDGET (bvw), -1);
+
+  return bvw->priv->speakersetup;
 }
 
 void
 bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
                                        BaconVideoWidgetAudioOutType type)
 {
-  
+  GstElement *audiosink = NULL;
+
+  g_return_if_fail (bvw != NULL);
+  g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+
+  if (type == bvw->priv->speakersetup)
+    return;
+
+  bvw->priv->speakersetup = type;
+  gconf_client_set_int (bvw->priv->gc,
+      GCONF_PREFIX"/audio_output_type", type, NULL);
+
+  g_object_get (G_OBJECT (bvw->priv->play), "audio-sink", &audiosink, NULL);
+  if (GST_STATE (bvw->priv->play) >= GST_STATE_PAUSED) {
+    //gst_pad_renegotiate (gst_element_get_pad (audiosink, "sink"));
+  }
 }
 
 /* =========================================== */
@@ -1140,8 +1489,8 @@ bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
 /* =========================================== */
 
 gboolean
-bacon_video_widget_open (BaconVideoWidget * bvw, const gchar * mrl,
-			 GError ** error)
+bacon_video_widget_open_with_subtitle (BaconVideoWidget * bvw,
+		const gchar * mrl, const gchar *subtitle_uri, GError ** error)
 {
   gboolean ret;
 
@@ -1171,17 +1520,30 @@ bacon_video_widget_open (BaconVideoWidget * bvw, const gchar * mrl,
   bvw->priv->media_has_video = FALSE;
   bvw->priv->stream_length = 0;
 
-  g_object_set (G_OBJECT (bvw->priv->play), "uri",
-		bvw->priv->mrl, NULL);
+  if (g_strrstr (bvw->priv->mrl, "#subtitle:")) {
+    gchar **uris;
+
+    uris = g_strsplit (bvw->priv->mrl, "#subtitle:", 2);
+    g_object_set (G_OBJECT (bvw->priv->play), "uri",
+                  uris[0], "suburi", uris[1], NULL);
+    g_strfreev (uris);
+  } else {
+    g_object_set (G_OBJECT (bvw->priv->play), "uri",
+		  bvw->priv->mrl, "suburi", subtitle_uri, NULL);
+  }
 
   ret = (gst_element_set_state (bvw->priv->play, GST_STATE_PAUSED) ==
       GST_STATE_SUCCESS);
   if (!ret)
     {
-      g_set_error (error, 0, 0, bvw->priv->last_error_message ?
+      g_set_error (error, 0, 0, "%s", bvw->priv->last_error_message ?
           bvw->priv->last_error_message : "Failed to open; reason unknown");
+      g_free (bvw->priv->mrl);
+      bvw->priv->mrl = NULL;
     }
 
+  g_signal_emit (bvw, bvw_table_signals[SIGNAL_CHANNELS_CHANGE], 0);
+
   return ret;
 }
 
@@ -1205,7 +1567,7 @@ bacon_video_widget_play (BaconVideoWidget * bvw, GError ** error)
       GST_STATE_PLAYING) == GST_STATE_SUCCESS);
   if (!ret)
     {
-      g_set_error (error, 0, 0, bvw->priv->last_error_message ?
+      g_set_error (error, 0, 0, "%s", bvw->priv->last_error_message ?
           bvw->priv->last_error_message : "Failed to play; reason unknown");
     }
 
@@ -1282,6 +1644,8 @@ bacon_video_widget_close (BaconVideoWidget * bvw)
     g_free (bvw->priv->mrl);
     bvw->priv->mrl = NULL;
   }
+
+  g_signal_emit (bvw, bvw_table_signals[SIGNAL_CHANNELS_CHANGE], 0);
 }
 
 void
@@ -1569,10 +1933,12 @@ setup_vis (BaconVideoWidget * bvw)
     }
   }
 
-  if (bvw->priv->show_vfx && bvw->priv->vis_element)
+  if (bvw->priv->show_vfx && bvw->priv->vis_element) {
+    gst_object_ref (GST_OBJECT (bvw->priv->vis_element));
     vis = bvw->priv->vis_element;
-  else
+  } else {
     vis = NULL;
+  }
 
   g_object_set (G_OBJECT (bvw->priv->play), "vis-plugin", vis, NULL);
 }
@@ -1586,6 +1952,8 @@ bacon_video_widget_set_show_visuals (BaconVideoWidget * bvw,
   g_return_val_if_fail (GST_IS_ELEMENT (bvw->priv->play), FALSE);
 
   bvw->priv->show_vfx = show_visuals;
+  gconf_client_set_bool (bvw->priv->gc,
+      GCONF_PREFIX"/enable_visualization", TRUE, NULL);
 
   setup_vis (bvw);
 
@@ -1676,6 +2044,9 @@ bacon_video_widget_set_visuals (BaconVideoWidget * bvw, const char *name)
   g_signal_connect (gst_element_get_pad (bvw->priv->vis_element, "src"),
 		    "fixate", G_CALLBACK (fixate_visualization), bvw);
 
+  gconf_client_set_string (bvw->priv->gc,
+      GCONF_PREFIX"/visualization_element", name, NULL);
+
   setup_vis (bvw);
   if (old_vis) {
     gst_object_unref (GST_OBJECT (old_vis));
@@ -1695,8 +2066,12 @@ bacon_video_widget_set_visuals_quality (BaconVideoWidget * bvw,
   if (bvw->priv->visq == quality)
     return;
   bvw->priv->visq = quality;
+  gconf_client_set_int (bvw->priv->gc,
+      GCONF_PREFIX"/visualization_quality", quality, NULL);
 
-  //gst_pad_renegotiate (gst_element_get_pad (bvw->priv->vis_element, "src"));
+  if (GST_STATE (bvw->priv->play) >= GST_STATE_PAUSED) {
+    //gst_pad_renegotiate (gst_element_get_pad (bvw->priv->vis_element, "src"));
+  }
 }
 
 gboolean
@@ -1905,6 +2280,9 @@ bacon_video_widget_set_video_property (BaconVideoWidget *bvw,
           g_object_unref (found_channel);
         }
     }
+
+  /* save in gconf */
+  gconf_client_set_int (bvw->priv->gc, video_props_str[type], value, NULL);
 }
 
 float
@@ -2187,6 +2565,7 @@ bacon_video_widget_get_metadata (BaconVideoWidget * bvw,
     case BVW_INFO_ALBUM:
     case BVW_INFO_VIDEO_CODEC:
     case BVW_INFO_AUDIO_CODEC:
+    case BVW_INFO_CDINDEX:
       bacon_video_widget_get_metadata_string (bvw, type, value);
       break;
     case BVW_INFO_DURATION:
@@ -2217,10 +2596,18 @@ bacon_video_widget_can_get_frames (BaconVideoWidget * bvw, GError ** error)
 
   /* check for version */
   if (!g_object_class_find_property (
-           G_OBJECT_GET_CLASS (bvw->priv->play), "frame"))
+           G_OBJECT_GET_CLASS (bvw->priv->play), "frame")) {
+    g_set_error (error, 0, 0,
+        "Too old version of GStreamer installed");
     return FALSE;
+  }
 
   /* check for video */
+  if (!bvw->priv->media_has_video) {
+    g_set_error (error, 0, 0,
+        "Media contains no supported video streams");
+  }
+
   return bvw->priv->media_has_video;
 }
 
@@ -2278,6 +2665,7 @@ bacon_video_widget_get_current_frame (BaconVideoWidget * bvw)
       g_assert (GST_IS_PAD (pad));
       real = (GstPad *) GST_PAD_REALIZE (pad);
       from = gst_caps_copy (GST_PAD_CAPS (real));
+      break;
     }
   }
   if (!from)
@@ -2329,6 +2717,12 @@ bacon_video_widget_get_popt_table (void)
   return (struct poptOption *) gst_init_get_popt_table ();
 }
 
+void
+bacon_video_widget_init_backend (int *argc, char ***argv)
+{
+  gst_init (argc, argv);
+}
+
 GQuark
 bacon_video_widget_error_quark (void)
 {
@@ -2353,9 +2747,11 @@ GtkWidget *
 bacon_video_widget_new (int width, int height,
 			BvwUseType type, GError ** err)
 {
+  GConfValue *confvalue;
   BaconVideoWidget *bvw;
   GstElement *audio_sink = NULL, *video_sink = NULL;
   gulong sig1, sig2;
+  gint i;
 
   bvw = BACON_VIDEO_WIDGET (g_object_new
                             (bacon_video_widget_get_type (), NULL));
@@ -2368,14 +2764,23 @@ bacon_video_widget_new (int width, int height,
     return NULL;
   }
   
+  bvw->priv->speakersetup = BVW_AUDIO_SOUND_STEREO;
   bvw->priv->media_device = g_strdup ("/dev/dvd");
   bvw->priv->init_width = 240;
   bvw->priv->init_height = 180;
+  bvw->priv->visq = VISUAL_SMALL;
+  bvw->priv->show_vfx = FALSE;
+  bvw->priv->vis_element = NULL;
+  bvw->priv->tv_out_type = TV_OUT_NONE;
+  bvw->priv->connection_speed = 0;
 
   bvw->priv->cursor_shown = TRUE;
   bvw->priv->logo_mode = TRUE;
   bvw->priv->auto_resize = TRUE;
 
+  /* gconf setting in backend */
+  bvw->priv->gc = gconf_client_get_default ();
+
   if (type == BVW_USE_TYPE_VIDEO || type == BVW_USE_TYPE_AUDIO) {
     audio_sink = gst_gconf_get_default_audio_sink ();
     if (!GST_IS_ELEMENT (audio_sink))
@@ -2387,11 +2792,38 @@ bacon_video_widget_new (int width, int height,
 
   if (type == BVW_USE_TYPE_VIDEO) {
     video_sink = gst_gconf_get_default_video_sink ();
-    if (!GST_IS_ELEMENT (video_sink))
-      {
+    if (!GST_IS_ELEMENT (video_sink)) {
         g_message ("failed to render default video sink from gconf");
         return NULL;
+    } else {
+      GDate d;
+
+      g_date_clear (&d, 1);
+      g_date_set_time (&d, time (NULL));
+      if (g_date_day (&d) == 1 && g_date_month (&d) == G_DATE_APRIL) {
+        confvalue = gconf_client_get_without_default (bvw->priv->gc,
+            GCONF_PREFIX"/puzzle_year", NULL);
+
+        if (!confvalue ||
+            gconf_value_get_int (confvalue) != g_date_year (&d)) {
+          GstElement *puzzle;
+
+          gconf_client_set_int (bvw->priv->gc, GCONF_PREFIX"/puzzle_year",
+              g_date_year (&d), NULL);
+
+          puzzle = gst_element_factory_make ("puzzle", NULL);
+          if (puzzle) {
+            GstElement *bin = gst_bin_new ("videosinkbin");
+
+            gst_bin_add_many (GST_BIN (bin), puzzle, video_sink, NULL);
+            gst_element_link (puzzle, video_sink);
+            gst_element_add_ghost_pad (bin,
+                gst_element_get_pad (puzzle, "sink"), "sink");
+            video_sink = bin;
+          }
+        }
       }
+    }
   }
 
   if (video_sink == NULL) {
@@ -2426,6 +2858,8 @@ bacon_video_widget_new (int width, int height,
 		video_sink, NULL);
   g_object_set (G_OBJECT (bvw->priv->play), "audio-sink",
 		audio_sink, NULL);
+  g_signal_connect (GST_PAD_REALIZE (gst_element_get_pad (audio_sink, "sink")),
+		    "fixate", G_CALLBACK (cb_audio_fixate), (gpointer) bvw);
 
   bvw->priv->vis_plugins_list = NULL;
 
@@ -2441,6 +2875,8 @@ bacon_video_widget_new (int width, int height,
 		    G_CALLBACK (got_buffering), (gpointer) bvw);
   g_signal_connect (G_OBJECT (bvw->priv->play), "notify::source",
 		    G_CALLBACK (got_source), (gpointer) bvw);
+  g_signal_connect (G_OBJECT (bvw->priv->play), "group-switch",
+		    G_CALLBACK (group_switch), (gpointer) bvw);
 
   /* We try to get an element supporting XOverlay interface */
   if (type == BVW_USE_TYPE_VIDEO && GST_IS_BIN (bvw->priv->play)) {
@@ -2495,6 +2931,60 @@ bacon_video_widget_new (int width, int height,
         bvw->priv->balance = GST_COLOR_BALANCE (element);
       }
   }
-  
+
+  /* Setup brightness and contrast */
+  for (i = 0; i < 4; i++) {
+    confvalue = gconf_client_get_without_default (bvw->priv->gc,
+        video_props_str[i], NULL);
+    if (confvalue != NULL) {
+      bacon_video_widget_set_video_property (bvw, i,
+        gconf_value_get_int (confvalue));
+    }
+  }
+
+  /* audio out, if any */
+  confvalue = gconf_client_get_without_default (bvw->priv->gc,
+      GCONF_PREFIX"/audio_output_type", NULL);
+  if (confvalue != NULL) {
+    bvw->priv->speakersetup = gconf_value_get_int (confvalue);
+  }
+
+  /* visualization */
+  confvalue = gconf_client_get_without_default (bvw->priv->gc,
+      GCONF_PREFIX"/enable_visualization", NULL);
+  if (confvalue != NULL) {
+    bvw->priv->show_vfx = gconf_value_get_bool (confvalue);
+  }
+  confvalue = gconf_client_get_without_default (bvw->priv->gc,
+      GCONF_PREFIX"/visualization_quality", NULL);
+  if (confvalue != NULL) {
+    bvw->priv->visq = gconf_value_get_int (confvalue);
+  }
+#if 0
+  confvalue = gconf_client_get_without_default (bvw->priv->gc,
+      GCONF_PREFIX"/visualization_element", NULL);
+  if (confvalue != NULL) {
+    bvw->priv->vis_element = 
+        gst_element_factory_make (gconf_value_get_string (confvalue), NULL);
+  }
+#endif
+  setup_vis (bvw);
+
+  /* tv/conn (not used yet) */
+  confvalue = gconf_client_get_without_default (bvw->priv->gc,
+      GCONF_PREFIX"/tv_out_type", NULL);
+  if (confvalue != NULL) {
+    bvw->priv->tv_out_type = gconf_value_get_int (confvalue);
+  }
+  confvalue = gconf_client_get_without_default (bvw->priv->gc,
+      GCONF_PREFIX"/connection_speed", NULL);
+  if (confvalue != NULL) {
+    bvw->priv->connection_speed = gconf_value_get_int (confvalue);
+  }
+
   return GTK_WIDGET (bvw);
 }
+
+/*
+ * vim: sw=2 ts=8 cindent noai bs=2
+ */
