@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2001-2002 the xine project
+ * Copyright (C) 2001-2005 the xine project
  * 	Heavily modified by Bastien Nocera <hadess@hadess.net>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.157 2005/01/03 12:50:18 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.163 2005/01/18 14:30:17 rbultje Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -188,19 +188,19 @@ static const char *mms_bandwidth_strs[] = {
 };
 
 static const char *audio_out_types_strs[] = {
-	"Mono",
-	"Stereo",
-	"Headphones",
-	"A52_Passthru",
-	"Surround21",
-	"Surround3",
-	"Surround4",
-	"Surround41",
-	"Surround5",
-	"Surround51",
-	"Surround6",
-	"Surround61",
-	"Surround71",
+	"Mono 1.0",
+	"Stereo 2.0",
+	"Headphones 2.0",
+	"Stereo 2.1",
+	"Surround 3.0",
+	"Surround 4.0",
+	"Surround 4.1",
+	"Surround 5.0",
+	"Surround 5.1",
+	"Surround 6.0",
+	"Surround 6.1",
+	"Surround 7.1",
+	"Pass Through",
 	NULL
 };
 
@@ -690,14 +690,14 @@ setup_config (BaconVideoWidget *bvw)
 
 	/* default demux strategy */
 	xine_config_register_enum (bvw->priv->xine,
-			"misc.demux_strategy",
+			"engine.demux.strategy",
 			0,
 			(char **) demux_strategies_str,
 			 "media format detection strategy",
 			 NULL, 10, NULL, NULL);
 
 	xine_config_lookup_entry (bvw->priv->xine,
-			"misc.demux_strategy", &entry);
+			"engine.demux.strategy", &entry);
 	entry.num_value = 0;
 	xine_config_update_entry (bvw->priv->xine, &entry);
 
@@ -706,7 +706,7 @@ setup_config (BaconVideoWidget *bvw)
 
 	/* Disable CDDB, we'll use Musicbrainz instead */
 	bvw_config_helper_num (bvw->priv->xine,
-			"input.cdda_use_cddb", 1, &entry);
+			"media.audio_cd.use_cddb", 1, &entry);
 	entry.num_value = 0;
 	xine_config_update_entry (bvw->priv->xine, &entry);
 
@@ -732,7 +732,7 @@ setup_config (BaconVideoWidget *bvw)
 	if (gconf_client_get_bool (bvw->priv->gc, "/system/http_proxy/use_http_proxy", NULL) == FALSE)
 	{
 		bvw_config_helper_string (bvw->priv->xine,
-				"input.http_proxy_host", "", &entry);
+				"media.network.http_proxy_host", "", &entry);
 		entry.str_value = "";
 		xine_config_update_entry (bvw->priv->xine, &entry);
 
@@ -740,13 +740,13 @@ setup_config (BaconVideoWidget *bvw)
 	}
 
 	bvw_config_helper_string (bvw->priv->xine,
-			"input.http_proxy_host", "", &entry);
+			"media.network.http_proxy_host", "", &entry);
 	entry.str_value = gconf_client_get_string (bvw->priv->gc,
 			"/system/http_proxy/host", NULL);
 	xine_config_update_entry (bvw->priv->xine, &entry);
 
 	bvw_config_helper_num (bvw->priv->xine,
-			 "input.http_proxy_port", 8080, &entry);
+			 "media.network.http_proxy_port", 8080, &entry);
 	entry.num_value = gconf_client_get_int (bvw->priv->gc,
 			"/system/http_proxy/port", NULL);
 	xine_config_update_entry (bvw->priv->xine, &entry);
@@ -754,19 +754,19 @@ setup_config (BaconVideoWidget *bvw)
 	if (gconf_client_get_bool (bvw->priv->gc, "/system/http_proxy/use_authentication", NULL) == FALSE)
 	{
 		bvw_config_helper_string (bvw->priv->xine,
-				"input.http_proxy_user", g_get_user_name(),
+				"media.network.http_proxy_user", g_get_user_name(),
 				&entry);
 		entry.str_value = "";
 		xine_config_update_entry (bvw->priv->xine, &entry);
 
 		bvw_config_helper_string (bvw->priv->xine,
-				"input.http_proxy_password", "",
+				"media.network.http_proxy_password", "",
 				&entry);
 		entry.str_value = "";
 		xine_config_update_entry (bvw->priv->xine, &entry);
 	} else {
 		bvw_config_helper_string (bvw->priv->xine,
-				"input.http_proxy_user", g_get_user_name(),
+				"media.network.http_proxy_user", g_get_user_name(),
 				&entry);
 		entry.str_value = gconf_client_get_string (bvw->priv->gc,
 				"/system/http_proxy/authentication_user",
@@ -774,7 +774,7 @@ setup_config (BaconVideoWidget *bvw)
 		xine_config_update_entry (bvw->priv->xine, &entry);
 
 		bvw_config_helper_string (bvw->priv->xine,
-				"input.http_proxy_password", "",
+				"media.network.http_proxy_password", "",
 				&entry);
 		entry.str_value = gconf_client_get_string (bvw->priv->gc,
 				"/system/http_proxy/authentication_password",
@@ -793,6 +793,9 @@ setup_config_stream (BaconVideoWidget *bvw)
 		return;
 
 	/* Setup brightness and contrast */
+	if (bvw->priv->vo_driver == NULL)
+		return;
+
 	for (i = 0; i < 4; i++)
 	{
 		confvalue = gconf_client_get_without_default (bvw->priv->gc,
@@ -865,7 +868,7 @@ dvd_skip_behaviour (BaconVideoWidget *bvw, int behaviour)
                 return;
 
         xine_config_register_num (bvw->priv->xine,
-                        "input.dvd_skip_behaviour",
+                        "media.dvd.skip_behaviour",
                         behaviour,
                         "DVD Skip behaviour",
                         NULL,
@@ -1512,6 +1515,12 @@ bacon_video_widget_get_popt_table (void)
 	return (struct poptOption *) xine_options;
 }
 
+void
+bacon_video_widget_init_backend (int *argc, char ***argv)
+{
+  /* no-op */
+}
+
 GQuark
 bacon_video_widget_error_quark (void)
 {
@@ -1539,7 +1548,7 @@ bacon_video_widget_new (int width, int height,
 	 * output */
 	if (type == BVW_USE_TYPE_VIDEO)
 	{
-		bvw_config_helper_num (bvw->priv->xine, "video.num_buffers",
+		bvw_config_helper_num (bvw->priv->xine, "engine.buffers.video_num_buffers",
 				500, &entry);
 		entry.num_value = 500;
 		xine_config_update_entry (bvw->priv->xine, &entry);
@@ -1587,7 +1596,7 @@ bacon_video_widget_new (int width, int height,
 		return NULL;
 	}
 
-	bvw_config_helper_num (bvw->priv->xine, "video.num_buffers",
+	bvw_config_helper_num (bvw->priv->xine, "engine.buffers.video_num_buffers",
 			5, &entry);
 	entry.num_value = 5;
 	xine_config_update_entry (bvw->priv->xine, &entry);
@@ -1841,66 +1850,17 @@ bacon_video_widget_get_backend_name (BaconVideoWidget *bvw)
 			XINE_SUB_VERSION);
 }
 
-/* List from xine-lib's demux_sputext.c */
-static const char *subtitle_ext[] = {
-	"asc",
-	"txt",
-	"sub",
-	"srt",
-	"smi",
-	"ssa"
-};
-
 static char *
-bacon_video_widget_get_subtitle_mrl (const char *mrl)
+bacon_video_widget_get_subtitled (const char *mrl, const char *subtitle_uri)
 {
-	char *suffix, *subtitle, *full;
-	guint len, i;
-
-	if (g_str_has_prefix (mrl, "file://") == FALSE) {
-		return NULL;
-	}
-
-	if (strstr (mrl, "#subtitle:") != NULL) {
-		return NULL;
-	}
-
-	len = strlen (mrl);
-	if (mrl[len-4] != '.') {
-		return NULL;
-	}
-
-	full = NULL;
-	subtitle = g_strdup (mrl);
-	suffix = subtitle + len - 4;
-	for (i = 0; i < G_N_ELEMENTS(subtitle_ext) ; i++) {
-		char *fname;
-
-		memcpy (suffix + 1, subtitle_ext[i], 3);
-		fname = g_filename_from_uri (subtitle, NULL, NULL);
-
-		if (fname == NULL)
-			continue;
-
-		if (g_file_test (fname, G_FILE_TEST_IS_REGULAR) == FALSE) {
-			g_free (fname);
-			continue;
-		}
-
-		full = g_strdup_printf ("%s#subtitle:%s", mrl,
-				subtitle + strlen ("file://"));
-		break;
-	}
-	g_free (subtitle);
-
-	return full;
+	g_return_val_if_fail (g_str_has_prefix (subtitle_uri, "file://"), NULL);
+	return g_strdup_printf ("%s#subtitle:%s", mrl, subtitle_uri + strlen ("file://"));
 }
 
 gboolean
-bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
-		GError **error)
+bacon_video_widget_open_with_subtitle (BaconVideoWidget *bvw, const char *mrl,
+		const char *subtitle_uri, GError **error)
 {
-	char *subtitled;
 	int err;
 
 	g_return_val_if_fail (mrl != NULL, FALSE);
@@ -1908,15 +1868,16 @@ bacon_video_widget_open (BaconVideoWidget *bvw, const char *mrl,
 	g_return_val_if_fail (bvw->priv->xine != NULL, FALSE);
 	g_return_val_if_fail (bvw->priv->mrl == NULL, FALSE);
 
-	subtitled = bacon_video_widget_get_subtitle_mrl (mrl);
-	if (subtitled != NULL) {
+	if (subtitle_uri != NULL)
+	{
+		char *subtitled;
+		subtitled = bacon_video_widget_get_subtitled (mrl, subtitle_uri);
 		err = xine_open (bvw->priv->stream, subtitled);
+		g_free (subtitled);
 	} else {
 		err = xine_open (bvw->priv->stream, mrl);
 	}
 
-	g_free (subtitled);
-
 	if (err == 0)
 	{
 		bacon_video_widget_close (bvw);
@@ -2462,25 +2423,25 @@ bacon_video_widget_set_media_device (BaconVideoWidget *bvw, const char *path)
 	xine_cfg_entry_t entry;
 
 	/* DVD device */
-	bvw_config_helper_string (bvw->priv->xine, "input.dvd_device",
+	bvw_config_helper_string (bvw->priv->xine, "media.dvd.device",
 			path, &entry);
 	entry.str_value = g_strdup (path);
 	xine_config_update_entry (bvw->priv->xine, &entry);
 
 	/* VCD device */
-	bvw_config_helper_string (bvw->priv->xine, "input.vcd_device",
+	bvw_config_helper_string (bvw->priv->xine, "media.vcd.device",
 			path, &entry);
 	entry.str_value = g_strdup (path);
 	xine_config_update_entry (bvw->priv->xine, &entry);
 
 	/* VCD device for the new input plugin */
-	bvw_config_helper_string (bvw->priv->xine, "vcd.default_device",
+	bvw_config_helper_string (bvw->priv->xine, "media.vcd.device",
 			path, &entry);
 	entry.str_value = g_strdup (path);
 	xine_config_update_entry (bvw->priv->xine, &entry);
 
 	/* CDDA device */
-	bvw_config_helper_string (bvw->priv->xine, "input.cdda_device",
+	bvw_config_helper_string (bvw->priv->xine, "media.audio_cd.device",
 			path, &entry);
 	entry.str_value = g_strdup (path);
 	xine_config_update_entry (bvw->priv->xine, &entry);
@@ -2493,12 +2454,12 @@ bacon_video_widget_set_proprietary_plugins_path (BaconVideoWidget *bvw,
 	xine_cfg_entry_t entry;
 
 	bvw_config_helper_string (bvw->priv->xine,
-			"codec.win32_path", path, &entry);
+			"decoder.external.win32_codecs_path", path, &entry);
 	entry.str_value = g_strdup (path);
 	xine_config_update_entry (bvw->priv->xine, &entry);
 
 	bvw_config_helper_string (bvw->priv->xine,
-			"codec.real_codecs_path", path, &entry);
+			"decoder.external.real_codecs_path", path, &entry);
 	entry.str_value = g_strdup (path);
 	xine_config_update_entry (bvw->priv->xine, &entry);
 
@@ -2524,14 +2485,14 @@ bacon_video_widget_set_connection_speed (BaconVideoWidget *bvw, int speed)
 	g_return_if_fail (speed > 0 || speed < 10);
 
 	xine_config_register_enum (bvw->priv->xine,
-			"input.mms_network_bandwidth",
+			"media.network.bandwidth",
 			6,
 			(char **) mms_bandwidth_strs,
 			"Network bandwidth",
 			NULL, 0, NULL, NULL);
 
 	xine_config_lookup_entry (bvw->priv->xine,
-			"input.mms_network_bandwidth", &entry);
+			"media.network.bandwidth", &entry);
 	entry.num_value = speed;
 	xine_config_update_entry (bvw->priv->xine, &entry);
 }
@@ -2546,14 +2507,14 @@ bacon_video_widget_get_connection_speed (BaconVideoWidget *bvw)
 	g_return_val_if_fail (bvw->priv->xine != NULL, 0);
 
 	xine_config_register_enum (bvw->priv->xine,
-			"input.mms_network_bandwidth",
+			"media.network.bandwidth",
 			6,
 			(char **) mms_bandwidth_strs,
 			"Network bandwidth",
 			NULL, 0, NULL, NULL);
 
 	xine_config_lookup_entry (bvw->priv->xine,
-			"input.mms_network_bandwidth", &entry);
+			"media.network.bandwidth", &entry);
 
 	return entry.num_value;
 }
@@ -2768,15 +2729,15 @@ bacon_video_widget_set_visuals_quality (BaconVideoWidget *bvw,
 		g_assert_not_reached ();
 	}
 
-	bvw_config_helper_num (bvw->priv->xine, "post.goom_fps", fps, &entry);
+	bvw_config_helper_num (bvw->priv->xine, "effects.goom.fps", fps, &entry);
 	entry.num_value = fps;
 	xine_config_update_entry (bvw->priv->xine, &entry);
 
-	bvw_config_helper_num (bvw->priv->xine, "post.goom_width", w, &entry);
+	bvw_config_helper_num (bvw->priv->xine, "effects.goom.width", w, &entry);
 	entry.num_value = w;
 	xine_config_update_entry (bvw->priv->xine, &entry);
 
-	bvw_config_helper_num (bvw->priv->xine, "post.goom_height", h, &entry);
+	bvw_config_helper_num (bvw->priv->xine, "effects.goom.height", h, &entry);
 	entry.num_value = h;
 	xine_config_update_entry (bvw->priv->xine, &entry);
 }
@@ -2936,7 +2897,7 @@ bacon_video_widget_set_video_device (BaconVideoWidget *bvw, const char *path)
 	xine_cfg_entry_t entry;
 
 	bvw_config_helper_string (bvw->priv->xine,
-			"input.v4l_video_device_path", path, &entry);
+			"media.video4linux.video_device", path, &entry);
 	entry.str_value = g_strdup (path);
 	xine_config_update_entry (bvw->priv->xine, &entry);
 }
@@ -3106,7 +3067,7 @@ bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
 	g_return_if_fail (bvw->priv->xine != NULL);
 
 	xine_config_register_enum (bvw->priv->xine,
-			"audio.speaker_arrangement",
+			"audio.output.speaker_arrangement",
 			1,
 			(char **) audio_out_types_strs,
 			"Speaker arrangement",
@@ -3121,19 +3082,19 @@ bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
 		value = 1;
 		break;
 	case BVW_AUDIO_SOUND_4CHANNEL:
-		value = 6;
+		value = 5;
 		break;
 	case BVW_AUDIO_SOUND_41CHANNEL:
-		value = 7;
+		value = 6;
 		break;
 	case BVW_AUDIO_SOUND_5CHANNEL:
-		value = 8;
+		value = 7;
 		break;
 	case BVW_AUDIO_SOUND_51CHANNEL:
-		value = 9;
+		value = 8;
 		break;
 	case BVW_AUDIO_SOUND_AC3PASSTHRU:
-		value = 3;
+		value = 12;
 		break;
 	default:
 		value = 1;
@@ -3141,7 +3102,7 @@ bacon_video_widget_set_audio_out_type (BaconVideoWidget *bvw,
 	}
 
 	xine_config_lookup_entry (bvw->priv->xine,
-			"audio.speaker_arrangement", &entry);
+			"audio.output.speaker_arrangement", &entry);
 	entry.num_value = value;
 	xine_config_update_entry (bvw->priv->xine, &entry);
 }
