@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* 
  * Copyright (C) 2001,2002,2003 Bastien Nocera <hadess@hadess.net>
  *
@@ -24,7 +25,7 @@
 #include <stdio.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <bacon-cd-selection.h>
+#include <nautilus-burn-drive-selection.h>
 
 #include "totem.h"
 #include "totem-private.h"
@@ -63,12 +64,6 @@ totem_action_info (char *reason, Totem *totem)
 	gtk_widget_show (error_dialog);
 }
 
-static void
-hide_prefs (GtkWidget *widget, int trash, Totem *totem)
-{
-	gtk_widget_hide (totem->prefs);
-}
-
 static gboolean
 ask_show_visuals (Totem *totem)
 {
@@ -80,9 +75,11 @@ ask_show_visuals (Totem *totem)
 				GTK_DIALOG_MODAL,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_YES_NO,
-				_("It seems you are running Totem remotely.\n"
-				"Are you sure you want to enable the visual "
-				"effects?"));
+				_("Enable visual effects?"));
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
+						  _("It seems you are running Totem remotely.\n"
+						    "Are you sure you want to enable the visual "
+						    "effects?"));
 	gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
 	gtk_dialog_set_default_response (GTK_DIALOG (dialog),
 			GTK_RESPONSE_NO);
@@ -115,12 +112,12 @@ totem_prefs_set_show_visuals (Totem *totem, gboolean value, gboolean warn)
 	item = glade_xml_get_widget (totem->xml, "tpw_visuals_type_label");
 	gtk_widget_set_sensitive (item, value);
 	item = glade_xml_get_widget (totem->xml,
-			"tpw_visuals_type_optionmenu");
+			"tpw_visuals_type_combobox");
 	gtk_widget_set_sensitive (item, value);
 	item = glade_xml_get_widget (totem->xml, "tpw_visuals_size_label");
 	gtk_widget_set_sensitive (item, value);
 	item = glade_xml_get_widget (totem->xml,
-			"tpw_visuals_size_optionmenu");
+			"tpw_visuals_size_combobox");
 	gtk_widget_set_sensitive (item, value);
 
 	if (warn == FALSE)
@@ -210,11 +207,11 @@ deinterlace_changed_cb (GConfClient *client, guint cnxn_id,
 }
 
 static void
-on_combo_entry1_changed (BaconCdSelection *bcs, char *device, Totem *totem)
+on_combo_entry1_changed (NautilusBurnDriveSelection *bcs, char *device, Totem *totem)
 {
 	const char *str;
 
-	str = bacon_cd_selection_get_device (bcs);
+	str = nautilus_burn_drive_selection_get_device (bcs);
 	gconf_client_set_string (totem->gc, GCONF_PREFIX"/mediadev", str, NULL);
 	bacon_video_widget_set_media_device
 		(BACON_VIDEO_WIDGET (totem->bvw), str);
@@ -273,7 +270,7 @@ mediadev_changed_cb (GConfClient *client, guint cnxn_id,
 	if (mediadev == NULL || strcmp (mediadev, "") == 0)
 		mediadev = g_strdup ("/dev/cdrom");
 
-	bacon_cd_selection_set_device (BACON_CD_SELECTION (item), mediadev);
+	nautilus_burn_drive_selection_set_device (NAUTILUS_BURN_DRIVE_SELECTION (item), mediadev);
 	bacon_video_widget_set_media_device
 		(BACON_VIDEO_WIDGET (totem->bvw), mediadev);
 
@@ -282,11 +279,11 @@ mediadev_changed_cb (GConfClient *client, guint cnxn_id,
 }
 
 static void
-option_menu_connection_changed (GtkOptionMenu *option_menu, Totem *totem)
+connection_combobox_changed (GtkComboBox *combobox, Totem *totem)
 {
 	int i;
 
-	i = gtk_option_menu_get_history (option_menu);
+	i = gtk_combo_box_get_active (combobox);
 	bacon_video_widget_set_connection_speed
 		(BACON_VIDEO_WIDGET (totem->bvw), i);
 }
@@ -318,14 +315,14 @@ on_button1_clicked (GtkButton *button, Totem *totem)
 }
 
 static void
-visual_menu_changed (GtkOptionMenu *option_menu, Totem *totem)
+visual_menu_changed (GtkComboBox *combobox, Totem *totem)
 {
 	GList *list;
 	const char *old_name;
 	char *name;
 	int i;
 
-	i = gtk_option_menu_get_history (GTK_OPTION_MENU (option_menu));
+	i = gtk_combo_box_get_active (combobox);
 	list = bacon_video_widget_get_visuals_list (totem->bvw);
 	name = g_list_nth_data (list, i);
 
@@ -343,11 +340,11 @@ visual_menu_changed (GtkOptionMenu *option_menu, Totem *totem)
 }
 
 static void
-visual_quality_menu_changed (GtkOptionMenu *option_menu, Totem *totem)
+visual_quality_menu_changed (GtkComboBox *combobox, Totem *totem)
 {
 	int i;
 
-	i = gtk_option_menu_get_history (GTK_OPTION_MENU (option_menu));
+	i = gtk_combo_box_get_active (combobox);
 	gconf_client_set_int (totem->gc,
 			GCONF_PREFIX"/visual_quality", i, NULL);
 	bacon_video_widget_set_visuals_quality (totem->bvw, i);
@@ -358,7 +355,7 @@ bacon_cd_selection_create (void)
 {
 	GtkWidget *widget;
 
-	widget = bacon_cd_selection_new ();
+	widget = nautilus_burn_drive_selection_new ();
 	gtk_widget_show (widget);
 
 	return widget;
@@ -423,10 +420,11 @@ on_tpw_color_reset_clicked (GtkButton *button, Totem *totem)
 }
 
 static void
-audio_out_menu_changed (GtkOptionMenu *option_menu, Totem *totem)
+audio_out_menu_changed (GtkComboBox *combobox, Totem *totem)
 {
 	BaconVideoWidgetAudioOutType audio_out;
-	audio_out = gtk_option_menu_get_history (GTK_OPTION_MENU (option_menu));
+
+	audio_out = gtk_combo_box_get_active (combobox);
 	bacon_video_widget_set_audio_out_type (totem->bvw, audio_out);
 }
 
@@ -435,7 +433,7 @@ totem_setup_preferences (Totem *totem)
 {
 	GtkWidget *item, *menu;
 	gboolean show_visuals, auto_resize, is_local, deinterlace;
-	int connection_speed, i, found;
+	int connection_speed, i;
 	char *path, *visual, *mediadev;
 	GList *list, *l;
 	BaconVideoWidgetAudioOutType audio_out;
@@ -444,7 +442,7 @@ totem_setup_preferences (Totem *totem)
 
 	is_local = totem_display_is_local ();
 
-	gconf_client_add_dir (totem->gc, "/apps/totem",
+	gconf_client_add_dir (totem->gc, GCONF_PREFIX,
 			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 	gconf_client_notify_add (totem->gc, GCONF_PREFIX"/auto_resize",
 			(GConfClientNotifyFunc) auto_resize_changed_cb,
@@ -456,12 +454,13 @@ totem_setup_preferences (Totem *totem)
 			(GConfClientNotifyFunc) mediadev_changed_cb,
 			totem, NULL, NULL);
 
-	totem->prefs = glade_xml_get_widget (totem->xml, "totem_preferences_window");
+	totem->prefs = glade_xml_get_widget (totem->xml,
+			"totem_preferences_window");
 
-	g_signal_connect (G_OBJECT (totem->prefs),
-			"response", G_CALLBACK (hide_prefs), (gpointer) totem);
+	g_signal_connect (G_OBJECT (totem->prefs), "response",
+			G_CALLBACK (gtk_widget_hide), NULL);
 	g_signal_connect (G_OBJECT (totem->prefs), "delete-event",
-			G_CALLBACK (hide_prefs), (gpointer) totem);
+			G_CALLBACK (gtk_widget_hide), NULL);
 
 	/* Auto-resize */
 	auto_resize = gconf_client_get_bool (totem->gc,
@@ -480,8 +479,9 @@ totem_setup_preferences (Totem *totem)
 	if (mediadev == NULL || (strcmp (mediadev, "") == 0)
 			|| (strcmp (mediadev, "auto") == 0))
 	{
-		mediadev = g_strdup (bacon_cd_selection_get_default_device
-				     (BACON_CD_SELECTION (item)));
+		mediadev = g_strdup
+			(nautilus_burn_drive_selection_get_default_device
+				     (NAUTILUS_BURN_DRIVE_SELECTION (item)));
 		gconf_client_set_string (totem->gc, GCONF_PREFIX"/mediadev",
 				mediadev, NULL);
 	}
@@ -491,16 +491,16 @@ totem_setup_preferences (Totem *totem)
 	g_signal_connect (G_OBJECT (item), "device-changed",
 			G_CALLBACK (on_combo_entry1_changed), totem);
 
-	bacon_cd_selection_set_device (BACON_CD_SELECTION (item), mediadev);
+	nautilus_burn_drive_selection_set_device
+		(NAUTILUS_BURN_DRIVE_SELECTION (item), mediadev);
 	g_free (mediadev);
 
 	/* Connection Speed */
 	connection_speed = bacon_video_widget_get_connection_speed (totem->bvw);
-	item = glade_xml_get_widget (totem->xml, "tpw_speed_optionmenu");
-	gtk_option_menu_set_history (GTK_OPTION_MENU (item),
-			connection_speed);
+	item = glade_xml_get_widget (totem->xml, "tpw_speed_combobox");
+	gtk_combo_box_set_active (GTK_COMBO_BOX (item), connection_speed);
 	g_signal_connect (item, "changed",
-			G_CALLBACK (option_menu_connection_changed), totem);
+			G_CALLBACK (connection_combobox_changed), totem);
 
 	/* Proprietary plugins */
 	item = glade_xml_get_widget (totem->xml, "tpw_plugins_button");
@@ -536,17 +536,19 @@ totem_setup_preferences (Totem *totem)
 	if (visual == NULL || strcmp (visual, "") == 0)
 		visual = g_strdup ("goom");
 
-	i = found = 0;
+	item = glade_xml_get_widget (totem->xml, "tpw_visuals_type_combobox");
+	g_signal_connect (G_OBJECT (item), "changed",
+			G_CALLBACK (visual_menu_changed), totem);
+
+	i = 0;
 	for (l = list; l != NULL; l = l->next)
 	{
 		const char *name = l->data;
 
-		item = gtk_menu_item_new_with_label (name);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		gtk_widget_show (item);
+		gtk_combo_box_append_text (GTK_COMBO_BOX (item), name);
 
 		if (strcmp (name, visual) == 0)
-			found = i;
+			gtk_combo_box_set_active (GTK_COMBO_BOX (item), i);
 
 		i++;
 	}
@@ -556,17 +558,11 @@ totem_setup_preferences (Totem *totem)
 	i = gconf_client_get_int (totem->gc,
 			GCONF_PREFIX"/visual_quality", NULL);
 	bacon_video_widget_set_visuals_quality (totem->bvw, i);
-	item = glade_xml_get_widget (totem->xml, "tpw_visuals_size_optionmenu");
-	gtk_option_menu_set_history (GTK_OPTION_MENU (item), i);
+	item = glade_xml_get_widget (totem->xml, "tpw_visuals_size_combobox");
+	gtk_combo_box_set_active (GTK_COMBO_BOX (item), i);
 	g_signal_connect (G_OBJECT (item), "changed",
 			G_CALLBACK (visual_quality_menu_changed), totem);
 
-	item = glade_xml_get_widget (totem->xml, "tpw_visuals_type_optionmenu");
-	gtk_option_menu_set_menu (GTK_OPTION_MENU (item), menu);
-	gtk_option_menu_set_history (GTK_OPTION_MENU (item), found);
-	g_signal_connect (G_OBJECT (item), "changed",
-			G_CALLBACK (visual_menu_changed), totem);
-
 	/* Brightness */
 	item = glade_xml_get_widget (totem->xml, "tpw_bright_scale");
 	i = bacon_video_widget_get_video_property (totem->bvw,
@@ -605,9 +601,9 @@ totem_setup_preferences (Totem *totem)
 			G_CALLBACK (on_tpw_color_reset_clicked), totem);
 
 	/* Sound output type */
-	item = glade_xml_get_widget (totem->xml, "tpw_sound_output_optionmenu");
+	item = glade_xml_get_widget (totem->xml, "tpw_sound_output_combobox");
 	audio_out = bacon_video_widget_get_audio_out_type (totem->bvw);
-	gtk_option_menu_set_history (GTK_OPTION_MENU (item), audio_out);
+	gtk_combo_box_set_active (GTK_COMBO_BOX (item), audio_out);
 	g_signal_connect (G_OBJECT (item), "changed",
 			G_CALLBACK (audio_out_menu_changed), totem);
 
